/* actioncode-System. Mehr dazu im Text weiter unten  */


enum
  {
    NO_ACTION,			/* actioncode = 0. Kein Befehl */
    ACTIVATE_TAB_GADGET,
    DISPLAY_NEXT_MESSAGE,
    DISPLAY_PREV_MESSAGE,
    DISPLAY_SELECTED_MESSAGE,	/* GetNewMsg() */
    SCROLL_DOWN_TEXTFIELD,
    SCROLL_UP_TEXTFIELD,
    QUIT_NEWSBREAKER,
    QUIT_NEWSBREAKER_SAFELY,	/* mit Abfrage vor Quit/Logout */
    REFRESH_WINDOW,
    GET_NEW_GROUP,
    OPEN_AMIGAGUIDE,
    OPEN_ABOUT,
    DISPLAY_HELP,
    DISPLAY_BUGS,
    DISPLAY_BUGREPORT,
    DISPLAY_ORDERFORM,
    DISPLAY_FEATUREWISH,
    CUT_TEXTFIELD,
    COPY_TEXTFIELD,
    COPYALL_TEXTFIELD,
    PASTE_TEXTFIELD,
    UNDO_TEXTFIELD,
    ERASE_TEXTFIELD,
    WRITE_REPLY_MESSAGE,
    WRITE_FOLLOWUP_MESSAGE,
    WRITE_EMAIL_MESSAGE,
    WRITE_POSTING_MESSAGE,
    GET_MAIL_GROUP,
    SET_FOCUS_ALL_MSG,
    SET_FOCUS_NEW_MSG,
    SET_FOCUS_LAST25_MSG,	/* fehlerhaft supported: gruppe < 25 msg --> 0 msg in liste! */
    DEBUG_SHOWUMSFIELD,
    DEBUG_SHOWGLOBALS,
    CATCHUP_CURRENT_GROUP,
    DELETE_MESSAGE_SAFELY,	/* lsche aktuelle Message, mit Sicherheitsabfrage */
    DELETE_MESSAGE,		/* lsche aktuelle Message, ohne Abfrage */
    SAVE_MESSAGE,
    PRINT_MESSAGE,
  };

/* Globale Variable */
ULONG actioncode = NO_ACTION;
ULONG lasterror = NO_ACTION;	/* fr spezielle Fehlerbehandlung */

/* Macro */
#define SETACTION(ac) actioncode = ac

/*
   ----------------------------------------------------
   Action Codes
   ----------------------------------------------------

   IDCMP Ereignisse systematisch auswerten. Ein kurzer berblick.


   A. Beschreibung
   B. Implementation
   C. Ausblick
   -----------------

   A.

   Jeder Actioncode ist mit einer NewsBreaker-Funktion verbunden.
   Z.B.
   PRINT_MESSAGE      --------> PrintUMSMessage()

   -- Userinput wird geholt
   -- Userinput wird ausgewertet
   -- ein Actioncode wird dem Userinput zugeordnet
   -- zum Actioncode passende Funktion wird gestartet


   B.

   Vorteile:
   1. spart Code und damit Programmieraufwand
   2. ist wiederverwertbar in anderen Projekten
   3. ist flexibel erweiterbar um neue Routinen und Actioncodes
   4. der Code ist unabhngig von den Eingabeverfahren. Ob Gadget,
   Men, Maus, Tastatur. Pro Eingabemethode gibt es ein
   Auswerteverfahren in wenigen Codezeilen. Spter wird es
   evtl. eine Ereignistabelle geben. (todo)

   - Im Menuitem steht "actioncode" in UserData
   - in der Gadget-Struktur ist "actioncode" in UserData
   - bei der Tastaturauswertung ist es noch switch/case-Konstrukt,
   hier wre eine Ereignistabelle ntzlich.

   C. Ausblick

   Zuknftige/Geplante Erweiterungen des Actioncode-Systems:

   - Ereignistabellen.
   - Dynamische, konfigurierbare Ereignisse whrend Laufzeit.
   - Parameterbergabe ist unabhngig von konkreten Funktionsparametern.
   -- z.B. als Tagliste
   -- z.B. ber DOS/ReadArgs()

spter: (todo) Zeiger auf erweiterte Actioncode-Struktur, als Input fr
typenfreie(?) Parameterbergabe im ReadArgs Template:

AUFRUFER
========

ActionCode
{
ULONG    ac_acode ;    -- Nummer der auszufhrenden Routine
STRPTR   ac_arguments; -- ReadArgs() kompatible Argumente
                          fr das zu acode definierte Template.
                          - Null, wenn default-Werte verwendet werden.
                          --Null, wenn kein Template vorhanden.
}

AUSWERTER
=========

   Zu jedem actioncode ein STRPTR auf ein ReadArgs-Template:
   z.B. bei
   actioncode                STRPTR ReadArgs-Template
   DELETE_MESSAGE_NR         "NUMMER/A"
   DELETE_CURRENT_MESSAGE    NULL

   (Textende Actioncodes)
 */

/* name/helpstr/guidenode sind auch default fr Button- und Menuhilfe */
/* todo : ActionCodeNode dokumentieren */

struct ActionCodeNode
  {
    struct Node ac_node;
    /*-------------------------------------------------------*/
    ULONG  ac_acode;		/* actioncode */
    STRPTR ac_template;         /* ReadArgs() Schablone */
    /*-------------------------------------------------------*/
    STRPTR name;		/* Funktionsbezeichner (i.d.R. Titel fr Onlinehilfefenster) */
    STRPTR helpstr;		/* kurzer Online-Hilfstext */
    STRPTR guidenode;		/* ausfhrliche Hilfe im Guide/guidenode */
    STRPTR command;             /* ARexx Kommandoname */
  };

/* VERALTET
ActioncodeInfo[] =
{
  {
    NO_ACTION, NULL, NULL, NULL, NULL, NULL
  }
  ,
  {
    ACTIVATE_TAB_GADGET, NULL, NULL, NULL, NULL, NULL
  }
  ,
  {
    DISPLAY_NEXT_MESSAGE, "Nchste Nachricht", NULL, NULL, NULL, NULL
  }
  ,
  {
    DISPLAY_PREV_MESSAGE, "Vorhergehende Nachricht", NULL, NULL, NULL, NULL
  }
  ,
  {
    DISPLAY_SELECTED_MESSAGE, "Ausgewhlte Nachricht", NULL, NULL, NULL, NULL
  }
  ,
  {
    SCROLL_DOWN_TEXTFIELD, "Text nach unten", NULL, NULL, NULL, NULL
  }
  ,
  {
    SCROLL_UP_TEXTFIELD, "Text nach oben", NULL, NULL, NULL, NULL
  }
  ,
  {
    QUIT_NEWSBREAKER, "Sofort beenden", NULL, NULL, NULL, NULL
  }
  ,
  {
    QUIT_NEWSBREAKER_SAFELY, "Beenden", NULL, NULL, NULL, NULL
  }
  ,
  {
    REFRESH_WINDOW, "Oberflche erneuern", NULL, NULL, NULL, NULL
  }
  ,
  {
    GET_NEW_GROUP, "Neue Gruppe", NULL, NULL, NULL, NULL
  }
  ,
  {
    OPEN_AMIGAGUIDE, "GUIDE ffnen", NULL, NULL, NULL, NULL
  }
  ,
  {
    CUT_TEXTFIELD, "Ausschneiden", NULL, NULL, NULL, NULL
  }
  ,
  {
    COPY_TEXTFIELD, "Kopieren", NULL, NULL, NULL, NULL
  }
  ,
  {
    COPYALL_TEXTFIELD, "Alles kopieren", NULL, NULL, NULL, NULL
  }
  ,
  {
    PASTE_TEXTFIELD, "Einfgen", NULL, NULL, NULL, NULL
  }
  ,
  {
    UNDO_TEXTFIELD, "Rckgngig machen", NULL, NULL, NULL, NULL
  }
  ,
  {
    ERASE_TEXTFIELD, "Lschen", NULL, NULL, NULL, NULL
  }
  ,
  {
    WRITE_REPLY_MESSAGE, "Reply", NULL, NULL, NULL, NULL
  }
  ,
  {
    WRITE_FOLLOWUP_MESSAGE, "Follow-Up", NULL, NULL, NULL, NULL
  }
  ,
  {
    WRITE_EMAIL_MESSAGE, "EMail", NULL, NULL, NULL, NULL
  }
  ,
  {
    WRITE_POSTING_MESSAGE, "Posting", NULL, NULL, NULL, NULL
  }
  ,
  {
    GET_MAIL_GROUP, "Hole Nachrichtenliste", NULL, NULL, NULL, NULL
  }
  ,
  {
    SET_FOCUS_ALL_MSG, "Alle Nachrichten", NULL, NULL, NULL, NULL
  }
  ,
  {
    SET_FOCUS_NEW_MSG, "Nur neue Nachrichten", NULL, NULL, NULL, NULL
  }
  ,
  {
    SET_FOCUS_LAST25_MSG, "Nur die 25 letzten", NULL, NULL, NULL, NULL
  }
  ,
  {
    DEBUG_SHOWUMSFIELD, "Zeige UMS-Felder", NULL, NULL, NULL, NULL
  }
  ,
  {
    DEBUG_SHOWGLOBALS, "Zeige Globale", NULL, NULL, NULL, NULL
  }
  ,
  {
    CATCHUP_CURRENT_GROUP, "Catch-Up", NULL, NULL, NULL, NULL
  }
  ,
  {
    DELETE_MESSAGE_SAFELY, "Nachricht lschen", NULL, NULL, NULL, NULL
  }
  ,
  {
    DELETE_MESSAGE, "Nachricht sofort lschen", NULL, NULL, NULL, NULL
  }
  ,
  {
    SAVE_MESSAGE, "Nachricht speichern", NULL, NULL, NULL, NULL
  }
  ,
  {
    PRINT_MESSAGE, "Nachricht drucken", NULL, NULL, NULL, NULL
  }
  ,
};
*/